//
// lempar.d - driver template for the LEMONade parser generator
//

private import std.stdio;
private import std.c.stdlib;

%%

%%

@ifndef INTERFACE
   @define INTERFACE 1
@endif

%%

const int YY_NO_ACTION     = (YYNSTATE+YYNRULE+2);
const int YY_ACCEPT_ACTION = (YYNSTATE+YYNRULE+1);
const int YY_ERROR_ACTION  = (YYNSTATE+YYNRULE);

%%

@ifdef YYFALLBACK
   static YYCODETYPE yyFallback[] =
   [
%%
   ];
@endif

struct yyStackEntry
{
  int stateno;          // The state-number
  int major;            // The major token value. This is the code
                        // number for the token at this stack level
  YYMINORTYPE minor;    // The user-supplied minor token value. (yylval in YACC)
                        // This is the value of the token
};

struct yyParser
{
  int yyidx;                           /* Index of top element in stack */
  int yyerrcnt;                        /* Shifts left before out of the error */
  ParseARG_SDECL                       /* A place to hold %extra_argument */
  yyStackEntry yystack[YYSTACKDEPTH];  /* The parser's stack */
};

debug(0)
{
   static FILE *yyTraceFILE = null;
   static char *yyTracePrompt = null;
}


debug(0)
{
   void ParseTrace( FILE *TraceFILE, char *TracePrompt )
   {
      yyTraceFILE = TraceFILE;
      yyTracePrompt = TracePrompt;
      if( yyTraceFILE == null )
         yyTracePrompt = null;
      else if( yyTracePrompt == "" )
         yyTraceFILE = null;
   }
}

debug(0)
{
   static const char *yyTokenName[] =
   [
%%
   ];
}

debug(0)
{
   static const char *yyRuleName[] =
   [
%%
   ];
}

char *ParseTokenName(int tokenType)
{
   debug(0)
   {
      if( tokenType > 0 && tokenType < yyTokenName.length )
         return yyTokenName[ tokenType ];
      else
         return "Unknown";
   }
   else
   {
      return "";
   }
}

yyParser *AllocParser( )
{
   yyParser *yyP;
   yyP = cast(yyParser*) malloc( yyParser.sizeof );
   if( yyP )
      yyP.yyidx = -1;
   return yyP;
}


static void yy_destructor( YYCODETYPE yymajor, YYMINORTYPE *yypminor )
{
   switch( yymajor )
   {
%%

      default:    // If no destructor action specified: do nothing
         break;
   }
}

static int yy_pop_parser_stack(yyParser *pParser)
{
   YYCODETYPE yymajor;
   yyStackEntry *yytos = &pParser.yystack[pParser.yyidx];

   if( pParser.yyidx<0 ) return 0;
   debug(0)
   {
      if( yyTraceFILE && pParser.yyidx>=0 )
      {
         fprintf(yyTraceFILE, "%sPopping %s\n", yyTracePrompt, yyTokenName[yytos.major] );
      }
   }

   yymajor = yytos.major;
   yy_destructor( yymajor, &yytos.minor );
   pParser.yyidx--;
   return yymajor;
}

void FreeParser( yyParser *ptr )
{
   if( ptr == null ) return;
   while( ptr.yyidx >= 0 )
      yy_pop_parser_stack(ptr);
   free(ptr);
}

static int yy_find_shift_action( yyParser *pParser, int iLookAhead )
{
   int i;
   int stateno = pParser.yystack[pParser.yyidx].stateno;

   /* if( pParser.>yyidx < 0 ) return YY_NO_ACTION;  ????????????? */
   i = yy_shift_ofst[stateno];
   if( i==YY_SHIFT_USE_DFLT )
   {
      return yy_default[stateno];
   }
   if( iLookAhead == YYNOCODE )
   {
      return YY_NO_ACTION;
   }
   i += iLookAhead;
   if( i < 0 || i >= yy_action.length || yy_lookahead[i] != iLookAhead )
   {
      @ifdef YYFALLBACK
      int iFallback;            /* Fallback token */
      if( iLookAhead<sizeof(yyFallback)/sizeof(yyFallback[0])
                 && (iFallback = yyFallback[iLookAhead])!=0 )
      {
         debug(0)
         {  if( yyTraceFILE )
            {  fprintf(yyTraceFILE, "%sFALLBACK %s => %s\n",
                     yyTracePrompt, yyTokenName[iLookAhead], yyTokenName[iFallback]);
            }
         }
         return yy_find_shift_action(pParser, iFallback);
      }
      @endif
      return yy_default[stateno];
   }
   else
   {
      return yy_action[i];
   }
}

/*
 * Find the appropriate action for a parser given the non-terminal
 * look-ahead token iLookAhead.
 *
 * If the look-ahead token is YYNOCODE, then check to see if the action is
 * independent of the look-ahead.  If it is, return the action, otherwise
 * return YY_NO_ACTION.
 */

static int yy_find_reduce_action(
  yyParser *pParser,                /* The parser */
  int iLookAhead )                  /* The look-ahead token */
{
   int i;
   int stateno = pParser.yystack[pParser.yyidx].stateno;

   i = yy_reduce_ofst[stateno];
   if (i == YY_REDUCE_USE_DFLT)
      return yy_default[stateno];

   if( iLookAhead==YYNOCODE )
      return YY_NO_ACTION;

   i += iLookAhead;
   if( i<0 || i>=yy_action.length || yy_lookahead[i]!=iLookAhead )
      return yy_default[stateno];
   else
      return yy_action[i];

}

/*
 *    Perform a shift action.
 */
static void yy_shift(
  yyParser *yypParser,          /* The parser to be shifted */
  int yyNewState,               /* The new state to shift in */
  int yyMajor,                  /* The major token to shift in */
  YYMINORTYPE *yypMinor )       /* Pointer ot the minor token to shift in */
{
   yyStackEntry *yytos;
   yypParser.yyidx++;
   if( yypParser.yyidx >= YYSTACKDEPTH )
   {
      yypParser.yyidx--;

      @ifndef NDEBUG
      if( yyTraceFILE )
      {
         fprintf(yyTraceFILE,"%sStack Overflow!\n",yyTracePrompt);
      }
      @endif

      while( yypParser.yyidx>=0 )
         yy_pop_parser_stack(yypParser);
      /* Here code is inserted which will execute if the parser stack ever overflows */
%%
      return;
   }

   yytos = &yypParser.yystack[yypParser.yyidx];
   yytos.stateno = yyNewState;
   yytos.major = yyMajor;
   yytos.minor = *yypMinor;

   @ifndef NDEBUG
   if( yyTraceFILE && yypParser.yyidx>0 )
   {
      int i;
      fprintf(yyTraceFILE,"%sShift %d\n",yyTracePrompt,yyNewState);
      fprintf(yyTraceFILE,"%sStack:",yyTracePrompt);
      for(i=1; i<=yypParser.yyidx; i++)
         fprintf(yyTraceFILE," %s",yyTokenName[yypParser.yystack[i].major]);
      fprintf(yyTraceFILE,"\n");
   }
   @endif

}

/* The following table contains information about every rule that
 * is used during the reduce.
 */

struct YYRULEINFO
{
   YYCODETYPE lhs;         /* Symbol on the left-hand side of the rule */
   ubyte nrhs;     /* Number of right-hand side symbols in the rule */
}

YYRULEINFO yyRuleInfo[] =
[
%%
];

//static void yy_accept(yyParser*);  /* Forward Declaration */

/*
 * Perform a reduce action and the shift that must immediately
 * follow the reduce.
 */

static void yy_reduce( yyParser *yypParser, int yyruleno )
{
   int yygoto;                     /* The next state */
   int yyact;                      /* The next action */
   YYMINORTYPE yygotominor;        /* The LHS of the rule reduced */
   yyStackEntry *yymsp;            /* The top of the parser's stack */
   int yysize;                     /* Amount to pop the stack */
   yymsp = &yypParser.yystack[yypParser.yyidx];

   @ifndef NDEBUG
   if( yyTraceFILE && yyruleno>=0 && yyruleno < yyRuleName.length )
   {
      fprintf(yyTraceFILE, "%sReduce [%s].\n", yyTracePrompt, yyRuleName[yyruleno]);
   }
   @endif

   switch( yyruleno )
   {
%%
      default:
         writef( "bug! \n" );
         break;
   };

   yygoto = yyRuleInfo[yyruleno].lhs;
   yysize = yyRuleInfo[yyruleno].nrhs;
   yypParser.yyidx -= yysize;
   yyact = yy_find_reduce_action(yypParser,yygoto);

   if( yyact < YYNSTATE )
      yy_shift(yypParser,yyact,yygoto,&yygotominor);
   else if( yyact == YYNSTATE + YYNRULE + 1 )
      yy_accept(yypParser);

}

static void yy_parse_failed( yyParser *yypParser )
{

   @ifndef NDEBUG
   if( yyTraceFILE )
   {
      fprintf(yyTraceFILE,"%sFail!\n",yyTracePrompt);
   }
   @endif

   while( yypParser.yyidx >= 0 )
      yy_pop_parser_stack( yypParser );

%%

}

static void yy_syntax_error( yyParser *yypParser, int yymajor, YYMINORTYPE yyminor )
{
   @define TOKEN (yyminor.yy0)

%%

}

static void yy_accept( yyParser *yypParser )
{

   @ifndef NDEBUG
   if( yyTraceFILE )
   {
      fprintf(yyTraceFILE,"%sAccept!\n",yyTracePrompt);
   }
   @endif
   while( yypParser.yyidx >= 0 )
      yy_pop_parser_stack(yypParser);

   /*
    * Here code is inserted which will be executed whenever the parser accepts
    */

%%

}

void Parse( void *yyp, int yymajor, ParseTOKENTYPE yyminor )
{
   YYMINORTYPE yyminorunion;
   int yyact;                    /* The parser action. */
   int yyendofinput;             /* True if we are at the end of input */
   int yyerrorhit = 0;           /* True if yymajor has invoked an error */
   yyParser *yypParser;          /* The parser */

   /* (re)initialize the parser, if necessary */
   yypParser = cast(yyParser*)yyp;
   if( yypParser.yyidx < 0 )
   {
      if( yymajor==0 ) return;
      yypParser.yyidx = 0;
      yypParser.yyerrcnt = -1;
      yypParser.yystack[0].stateno = 0;
      yypParser.yystack[0].major = 0;
   }
   yyminorunion.yy0 = yyminor;
   yyendofinput = (yymajor==0);

   @ifndef NDEBUG
   if( yyTraceFILE )
   {
      fprintf(yyTraceFILE,"%sInput %s\n",yyTracePrompt,yyTokenName[yymajor]);
   }
   @endif

   do
   {
      yyact = yy_find_shift_action(yypParser,yymajor);
      if( yyact < YYNSTATE )
      {
         yy_shift(yypParser,yyact,yymajor,&yyminorunion);
         yypParser.yyerrcnt--;
         if( yyendofinput && yypParser.yyidx>=0 )
         {
            yymajor = 0;
         }
         else
         {
            yymajor = YYNOCODE;
         }
      }
      else if( yyact < YYNSTATE + YYNRULE )
      {
         yy_reduce(yypParser,yyact-YYNSTATE);
      }
      else if( yyact == YY_ERROR_ACTION )
      {
         int yymx;

         @ifndef NDEBUG
         if( yyTraceFILE )
         {
            fprintf(yyTraceFILE,"%sSyntax Error!\n",yyTracePrompt);
         }
         @endif

         @ifdef YYERRORSYMBOL
         if( yypParser.yyerrcnt < 0 )
         {
            yy_syntax_error(yypParser,yymajor,yyminorunion);
         }
         yymx = yypParser.yystack[yypParser.yyidx].major;
         if( yymx==YYERRORSYMBOL || yyerrorhit )
         {
            @ifndef NDEBUG
            if( yyTraceFILE )
            {
               fprintf(yyTraceFILE,"%sDiscard input token %s\n",
                  yyTracePrompt,yyTokenName[yymajor]);
            }
            @endif
         yy_destructor(yymajor,&yyminorunion);
         yymajor = YYNOCODE;
         }
         else
         {
            while( yypParser.yyidx >= 0 && yymx != YYERRORSYMBOL &&
               (yyact = yy_find_shift_action(yypParser,YYERRORSYMBOL)) >= YYNSTATE )
            {
               yy_pop_parser_stack(yypParser);
            }
            if( yypParser.yyidx < 0 || yymajor==0 )
            {
               yy_destructor(yymajor,&yyminorunion);
               yy_parse_failed(yypParser);
               yymajor = YYNOCODE;
            }
            else if( yymx!=YYERRORSYMBOL )
            {
               YYMINORTYPE u2;
               u2.YYERRSYMDT = 0;
               yy_shift(yypParser,yyact,YYERRORSYMBOL,&u2);
            }
         }
         yypParser.yyerrcnt = 3;
         yyerrorhit = 1;

         @else  /* YYERRORSYMBOL is not defined */

         if( yypParser.yyerrcnt<=0 )
         {
            yy_syntax_error(yypParser,yymajor,yyminorunion);
         }
         yypParser.yyerrcnt = 3;
         yy_destructor(yymajor,&yyminorunion);
         if( yyendofinput )
         {
            yy_parse_failed(yypParser);
         }
         yymajor = YYNOCODE;
         @endif
      }
      else
      {
         yy_accept(yypParser);
         yymajor = YYNOCODE;
      }
   }  while( yymajor!=YYNOCODE && yypParser.yyidx >= 0 );
   return;
}

/**/
